# coding: utf-8

"""
    API Reference

    Detailed API reference documentation for Visier APIs. Includes all endpoints, headers, path parameters, query parameters, request body schema, response schema, JSON request samples, and JSON response samples.

    The version of the OpenAPI document: 22222222.99201.2042
    Contact: alpine@visier.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class ReportDTO(BaseModel):
    """
    The information of a single report.
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, description="The unique ID of the report.")
    title: Optional[StrictStr] = Field(default=None, description="The localized title of the report.")
    description: Optional[StrictStr] = Field(default=None, description="The localized description of the report.")
    updated_time: Optional[StrictStr] = Field(default=None, description="The time in epoch milliseconds when the report was last updated.", alias="updatedTime")
    is_published_report: Optional[StrictBool] = Field(default=None, description="If `true`, the report is published through a project.", alias="isPublishedReport")
    owner: Optional[StrictStr] = Field(default=None, description="The owner of the report.")
    is_owned_by_user: Optional[StrictBool] = Field(default=None, description="If `true`, you are the owner of the report.", alias="isOwnedByUser")
    allowed_actions: Optional[List[StrictStr]] = Field(default=None, description="The actions that you can perform on the report.   Valid values are:  - `duplicate`: The user can make their own copy of this report.  - `delete`: The user can delete this report.  - `edit`: The user can edit this report directly.  - `download`: The user can download this report as a CSV.  - `share`: The user can share this report with other users or user groups.", alias="allowedActions")
    link: Optional[StrictStr] = Field(default=None, description="The link to the report.")
    __properties: ClassVar[List[str]] = ["id", "title", "description", "updatedTime", "isPublishedReport", "owner", "isOwnedByUser", "allowedActions", "link"]

    @field_validator('allowed_actions')
    def allowed_actions_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        for i in value:
            if i not in set(['duplicate', 'delete', 'edit', 'download', 'share']):
                raise ValueError("each list item must be one of ('duplicate', 'delete', 'edit', 'download', 'share')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ReportDTO from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ReportDTO from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "title": obj.get("title"),
            "description": obj.get("description"),
            "updatedTime": obj.get("updatedTime"),
            "isPublishedReport": obj.get("isPublishedReport"),
            "owner": obj.get("owner"),
            "isOwnedByUser": obj.get("isOwnedByUser"),
            "allowedActions": obj.get("allowedActions"),
            "link": obj.get("link")
        })
        return _obj


